@@ -114,6 +114,7 @@ public class ImportHandler {
         servletHttpClassNames.add("HttpSessionEvent");
         servletHttpClassNames.add("HttpUtils");
         // Enums
+        servletHttpClassNames.add("MappingMatch");
         standardPackages.put("javax.servlet.http", servletHttpClassNames);
 
         // JSP 2.3
@@ -136,7 +137,7 @@ public class ImportHandler {
         standardPackages.put("javax.servlet.jsp", servletJspClassNames);
 
         Set<String> javaLangClassNames = new HashSet<>();
-        // Based on Java 21 EA24
+        // Based on Java 26 EA26
         // Interfaces
         javaLangClassNames.add("Appendable");
         javaLangClassNames.add("AutoCloseable");
@@ -144,6 +145,7 @@ public class ImportHandler {
         javaLangClassNames.add("Cloneable");
         javaLangClassNames.add("Comparable");
         javaLangClassNames.add("Iterable");
+        javaLangClassNames.add("LazyConstant");
         javaLangClassNames.add("ProcessHandle");
         javaLangClassNames.add("ProcessHandle.Info");
         javaLangClassNames.add("Readable");
@@ -168,6 +170,7 @@ public class ImportHandler {
         javaLangClassNames.add("Enum");
         javaLangClassNames.add("Enum.EnumDesc");
         javaLangClassNames.add("Float");
+        javaLangClassNames.add("IO");
         javaLangClassNames.add("InheritableThreadLocal");
         javaLangClassNames.add("Integer");
         javaLangClassNames.add("Long");
@@ -185,8 +188,12 @@ public class ImportHandler {
         javaLangClassNames.add("Runtime");
         javaLangClassNames.add("Runtime.Version");
         javaLangClassNames.add("RuntimePermission");
+        javaLangClassNames.add("ScopedValue");
+        javaLangClassNames.add("ScopedValue.CallableOp");
+        javaLangClassNames.add("ScopedValue.Carrier");
         javaLangClassNames.add("SecurityManager");
         javaLangClassNames.add("Short");
+        javaLangClassNames.add("StableValue");
         javaLangClassNames.add("StackTraceElement");
         javaLangClassNames.add("StackWalker");
         javaLangClassNames.add("StrictMath");
@@ -276,7 +283,7 @@ public class ImportHandler {
     }
 
     private Map<String,Set<String>> packageNames = new ConcurrentHashMap<>();
-    private ConcurrentHashMap<String,String> classNames = new ConcurrentHashMap<>();
+    private Map<String,String> classNames = new ConcurrentHashMap<>();
     private Map<String,Class<?>> clazzes = new ConcurrentHashMap<>();
     private Map<String,Class<?>> statics = new ConcurrentHashMap<>();
 
@@ -286,7 +293,7 @@ public class ImportHandler {
     }
 
 
-    public void importStatic(String name) throws javax.el.ELException {
+    public void importStatic(String name) throws ELException {
         int lastPeriod = name.lastIndexOf('.');
 
         if (lastPeriod < 0) {
@@ -341,7 +348,7 @@ public class ImportHandler {
     }
 
 
-    public void importClass(String name) throws javax.el.ELException {
+    public void importClass(String name) throws ELException {
         int lastPeriodIndex = name.lastIndexOf('.');
 
         if (lastPeriodIndex < 0) {
@@ -366,13 +373,14 @@ public class ImportHandler {
         // c) such check is not required by specification.
         Set<String> preloaded = standardPackages.get(name);
         if (preloaded == null) {
-            preloaded = Collections.emptySet();
+            packageNames.put(name, Collections.emptySet());
+        } else {
+            packageNames.put(name, preloaded);
         }
-        packageNames.put(name, preloaded);
     }
 
 
-    public java.lang.Class<?> resolveClass(String name) {
+    public Class<?> resolveClass(String name) {
         if (name == null || name.contains(".")) {
             return null;
         }
@@ -396,6 +404,18 @@ public class ImportHandler {
                 clazzes.put(name, clazz);
                 return clazz;
             }
+            // Might be an inner class
+            StringBuilder sb = new StringBuilder(className);
+            int replacementPosition = sb.lastIndexOf(".");
+            while (replacementPosition > -1) {
+                sb.setCharAt(replacementPosition, '$');
+                clazz = findClass(sb.toString(), true);
+                if (clazz != null) {
+                    clazzes.put(name, clazz);
+                    return clazz;
+                }
+                replacementPosition = sb.lastIndexOf(".", replacementPosition);
+            }
         }
 
         // Search the package imports - note there may be multiple matches
@@ -432,7 +452,7 @@ public class ImportHandler {
     }
 
 
-    public java.lang.Class<?> resolveStatic(String name) {
+    public Class<?> resolveStatic(String name) {
         return statics.get(name);
     }
 
