@@ -42,8 +42,6 @@ import org.apache.tomcat.util.net.SSLHostConfig.CertificateVerification;
 /**
  * An <b>Authenticator</b> and <b>Valve</b> implementation of authentication that utilizes SSL certificates to identify
  * client users.
- *
- * @author Craig R. McClanahan
  */
 public class SSLAuthenticator extends AuthenticatorBase {
 
@@ -61,14 +59,17 @@ public class SSLAuthenticator extends AuthenticatorBase {
     @Override
     protected boolean doAuthenticate(Request request, HttpServletResponse response) throws IOException {
 
-        // NOTE: We don't try to reauthenticate using any existing SSO session,
-        // because that will only work if the original authentication was
-        // BASIC or FORM, which are less secure than the CLIENT-CERT auth-type
-        // specified for this webapp
-        //
-        // Change to true below to allow previous FORM or BASIC authentications
-        // to authenticate users for this webapp
-        // TODO make this a configurable attribute (in SingleSignOn??)
+        /*
+         * Reauthentication using the cached user name and password (if any) is not enabled for CLIENT-CERT
+         * authentication. This was an historical design decision made because CLIENT-CERT authentication is viewed as
+         * more secure than BASIC/FORM.
+         *
+         * However, reauthentication was introduced to handle the case where the Realm took additional actions on
+         * authentication. Reauthenticating with the cached user name and password may not be sufficient for CLIENT-CERT
+         * since it will not make any TLS information (client certificate etc) available that a web application may
+         * depend on. Therefore, the reauthentication behaviour for CLIENT-CERT is to perform a normal CLIENT-CERT
+         * authentication.
+         */
         if (checkForCachedAuthentication(request, response, false)) {
             return true;
         }
@@ -203,7 +204,8 @@ public class SSLAuthenticator extends AuthenticatorBase {
                 UpgradeProtocol[] upgradeProtocols = connector.findUpgradeProtocols();
                 for (UpgradeProtocol upgradeProtocol : upgradeProtocols) {
                     if ("h2".equals(upgradeProtocol.getAlpnName())) {
-                        log.warn(sm.getString("sslAuthenticatorValve.http2", context.getName(), host.getName(), connector));
+                        log.warn(sm.getString("sslAuthenticatorValve.http2", context.getName(), host.getName(),
+                                connector));
                         break;
                     }
                 }
