@@ -34,21 +34,17 @@ import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
 
 /**
- * <p>Utility class for building class loaders for Catalina.  The factory
- * method requires the following parameters in order to build a new class
- * loader (with suitable defaults in all cases):</p>
+ * <p>
+ * Utility class for building class loaders for Catalina. The factory method requires the following parameters in order
+ * to build a new class loader (with suitable defaults in all cases):
+ * </p>
  * <ul>
- * <li>A set of directories containing unpacked classes (and resources)
- *     that should be included in the class loader's
- *     repositories.</li>
- * <li>A set of directories containing classes and resources in JAR files.
- *     Each readable JAR file discovered in these directories will be
- *     added to the class loader's repositories.</li>
- * <li><code>ClassLoader</code> instance that should become the parent of
- *     the new class loader.</li>
+ * <li>A set of directories containing unpacked classes (and resources) that should be included in the class loader's
+ * repositories.</li>
+ * <li>A set of directories containing classes and resources in JAR files. Each readable JAR file discovered in these
+ * directories will be added to the class loader's repositories.</li>
+ * <li><code>ClassLoader</code> instance that should become the parent of the new class loader.</li>
  * </ul>
- *
- * @author Craig R. McClanahan
  */
 public final class ClassLoaderFactory {
 
@@ -59,25 +55,20 @@ public final class ClassLoaderFactory {
 
 
     /**
-     * Create and return a new class loader, based on the configuration
-     * defaults and the specified directory paths:
+     * Create and return a new class loader, based on the configuration defaults and the specified directory paths:
+     *
+     * @param unpacked Array of pathnames to unpacked directories that should be added to the repositories of the class
+     *                     loader, or <code>null</code> for no unpacked directories to be considered
+     * @param packed   Array of pathnames to directories containing JAR files that should be added to the repositories
+     *                     of the class loader, or <code>null</code> for no directories of JAR files to be considered
+     * @param parent   Parent class loader for the new class loader, or <code>null</code> for the system class loader.
      *
-     * @param unpacked Array of pathnames to unpacked directories that should
-     *  be added to the repositories of the class loader, or <code>null</code>
-     * for no unpacked directories to be considered
-     * @param packed Array of pathnames to directories containing JAR files
-     *  that should be added to the repositories of the class loader,
-     * or <code>null</code> for no directories of JAR files to be considered
-     * @param parent Parent class loader for the new class loader, or
-     *  <code>null</code> for the system class loader.
      * @return the new class loader
      *
      * @exception Exception if an error occurs constructing the class loader
      */
-    public static ClassLoader createClassLoader(File unpacked[],
-                                                File packed[],
-                                                final ClassLoader parent)
-        throws Exception {
+    public static ClassLoader createClassLoader(File[] unpacked, File[] packed, final ClassLoader parent)
+            throws Exception {
 
         if (log.isDebugEnabled()) {
             log.debug("Creating new class loader");
@@ -107,7 +98,7 @@ public final class ClassLoaderFactory {
                 if (!directory.isDirectory() || !directory.canRead()) {
                     continue;
                 }
-                String filenames[] = directory.list();
+                String[] filenames = directory.list();
                 if (filenames == null) {
                     continue;
                 }
@@ -128,33 +119,30 @@ public final class ClassLoaderFactory {
 
         // Construct the class loader itself
         final URL[] array = set.toArray(new URL[0]);
-        return AccessController.doPrivileged(
-                (PrivilegedAction<URLClassLoader>) () -> {
-                    if (parent == null) {
-                        return new URLClassLoader(array);
-                    } else {
-                        return new URLClassLoader(array, parent);
-                    }
-                });
+        return AccessController.doPrivileged((PrivilegedAction<URLClassLoader>) () -> {
+            if (parent == null) {
+                return new URLClassLoader(array);
+            } else {
+                return new URLClassLoader(array, parent);
+            }
+        });
     }
 
 
     /**
-     * Create and return a new class loader, based on the configuration
-     * defaults and the specified directory paths:
+     * Create and return a new class loader, based on the configuration defaults and the specified directory paths:
+     *
+     * @param repositories List of class directories, jar files, jar directories or URLS that should be added to the
+     *                         repositories of the class loader.
+     * @param parent       Parent class loader for the new class loader, or <code>null</code> for the system class
+     *                         loader.
      *
-     * @param repositories List of class directories, jar files, jar directories
-     *                     or URLS that should be added to the repositories of
-     *                     the class loader.
-     * @param parent Parent class loader for the new class loader, or
-     *  <code>null</code> for the system class loader.
      * @return the new class loader
      *
      * @exception Exception if an error occurs constructing the class loader
      */
-    public static ClassLoader createClassLoader(List<Repository> repositories,
-                                                final ClassLoader parent)
-        throws Exception {
+    public static ClassLoader createClassLoader(List<Repository> repositories, final ClassLoader parent)
+            throws Exception {
 
         if (log.isDebugEnabled()) {
             log.debug("Creating new class loader");
@@ -164,7 +152,7 @@ public final class ClassLoaderFactory {
         Set<URL> set = new LinkedHashSet<>();
 
         if (repositories != null) {
-            for (Repository repository : repositories)  {
+            for (Repository repository : repositories) {
                 if (repository.getType() == RepositoryType.URL) {
                     URL url = buildClassLoaderUrl(repository.getLocation());
                     if (log.isDebugEnabled()) {
@@ -183,7 +171,7 @@ public final class ClassLoaderFactory {
                     }
                     set.add(url);
                 } else if (repository.getType() == RepositoryType.JAR) {
-                    File file=new File(repository.getLocation());
+                    File file = new File(repository.getLocation());
                     file = file.getCanonicalFile();
                     if (!validateFile(file, RepositoryType.JAR)) {
                         continue;
@@ -194,16 +182,15 @@ public final class ClassLoaderFactory {
                     }
                     set.add(url);
                 } else if (repository.getType() == RepositoryType.GLOB) {
-                    File directory=new File(repository.getLocation());
+                    File directory = new File(repository.getLocation());
                     directory = directory.getCanonicalFile();
                     if (!validateFile(directory, RepositoryType.GLOB)) {
                         continue;
                     }
                     if (log.isDebugEnabled()) {
-                        log.debug("  Including directory glob "
-                            + directory.getAbsolutePath());
+                        log.debug("  Including directory glob " + directory.getAbsolutePath());
                     }
-                    String filenames[] = directory.list();
+                    String[] filenames = directory.list();
                     if (filenames == null) {
                         continue;
                     }
@@ -218,8 +205,7 @@ public final class ClassLoaderFactory {
                             continue;
                         }
                         if (log.isDebugEnabled()) {
-                            log.debug("    Including glob jar file "
-                                    + file.getAbsolutePath());
+                            log.debug("    Including glob jar file " + file.getAbsolutePath());
                         }
                         URL url = buildClassLoaderUrl(file);
                         set.add(url);
@@ -230,43 +216,38 @@ public final class ClassLoaderFactory {
 
         // Construct the class loader itself
         final URL[] array = set.toArray(new URL[0]);
-        if (log.isDebugEnabled()) {
+        if (log.isTraceEnabled()) {
             for (int i = 0; i < array.length; i++) {
-                log.debug("  location " + i + " is " + array[i]);
+                log.trace("  location " + i + " is " + array[i]);
             }
         }
 
-        return AccessController.doPrivileged(
-                (PrivilegedAction<URLClassLoader>) () -> {
-                    if (parent == null) {
-                        return new URLClassLoader(array);
-                    } else {
-                        return new URLClassLoader(array, parent);
-                    }
-                });
+        return AccessController.doPrivileged((PrivilegedAction<URLClassLoader>) () -> {
+            if (parent == null) {
+                return new URLClassLoader(array);
+            } else {
+                return new URLClassLoader(array, parent);
+            }
+        });
     }
 
-    private static boolean validateFile(File file,
-            RepositoryType type) throws IOException {
+    private static boolean validateFile(File file, RepositoryType type) throws IOException {
         if (RepositoryType.DIR == type || RepositoryType.GLOB == type) {
             if (!file.isDirectory() || !file.canRead()) {
-                String msg = "Problem with directory [" + file +
-                        "], exists: [" + file.exists() +
-                        "], isDirectory: [" + file.isDirectory() +
-                        "], canRead: [" + file.canRead() + "]";
+                String msg = "Problem with directory [" + file + "], exists: [" + file.exists() + "], isDirectory: [" +
+                        file.isDirectory() + "], canRead: [" + file.canRead() + "]";
 
-                File home = new File (Bootstrap.getCatalinaHome());
+                File home = new File(Bootstrap.getCatalinaHome());
                 home = home.getCanonicalFile();
-                File base = new File (Bootstrap.getCatalinaBase());
+                File base = new File(Bootstrap.getCatalinaBase());
                 base = base.getCanonicalFile();
                 File defaultValue = new File(base, "lib");
 
                 // Existence of ${catalina.base}/lib directory is optional.
                 // Hide the warning if Tomcat runs with separate catalina.home
                 // and catalina.base and that directory is absent.
-                if (!home.getPath().equals(base.getPath())
-                        && file.getPath().equals(defaultValue.getPath())
-                        && !file.exists()) {
+                if (!home.getPath().equals(base.getPath()) && file.getPath().equals(defaultValue.getPath()) &&
+                        !file.exists()) {
                     log.debug(msg);
                 } else {
                     log.warn(msg);
@@ -275,9 +256,8 @@ public final class ClassLoaderFactory {
             }
         } else if (RepositoryType.JAR == type) {
             if (!file.canRead()) {
-                log.warn("Problem with JAR file [" + file +
-                        "], exists: [" + file.exists() +
-                        "], canRead: [" + file.canRead() + "]");
+                log.warn("Problem with JAR file [" + file + "], exists: [" + file.exists() + "], canRead: [" +
+                        file.canRead() + "]");
                 return false;
             }
         }
@@ -286,9 +266,8 @@ public final class ClassLoaderFactory {
 
 
     /*
-     * These two methods would ideally be in the utility class
-     * org.apache.tomcat.util.buf.UriUtil but that class is not visible until
-     * after the class loaders have been constructed.
+     * These two methods would ideally be in the utility class org.apache.tomcat.util.buf.UriUtil but that class is not
+     * visible until after the class loaders have been constructed.
      */
     private static URL buildClassLoaderUrl(String urlString) throws MalformedURLException, URISyntaxException {
         // URLs passed to class loaders may point to directories that contain
