@@ -36,9 +36,9 @@ import javax.net.ssl.TrustManagerFactory;
 
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
-import org.apache.tomcat.util.compat.JreCompat;
 import org.apache.tomcat.util.net.openssl.OpenSSLConf;
 import org.apache.tomcat.util.net.openssl.ciphers.Cipher;
+import org.apache.tomcat.util.net.openssl.ciphers.Group;
 import org.apache.tomcat.util.net.openssl.ciphers.OpenSSLCipherConfigurationParser;
 import org.apache.tomcat.util.res.StringManager;
 
@@ -52,16 +52,23 @@ public class SSLHostConfig implements Serializable {
     private static final Log log = LogFactory.getLog(SSLHostConfig.class);
     private static final StringManager sm = StringManager.getManager(SSLHostConfig.class);
 
-    // Must be lower case. SSL host names are always stored using lower case as
-    // they are case insensitive but are used by case sensitive code such as
+    // Must be lowercase. SSL host names are always stored using lower case as
+    // they are case-insensitive but are used by case-sensitive code such as
     // keys in Maps.
     protected static final String DEFAULT_SSL_HOST_NAME = "_default_";
     protected static final Set<String> SSL_PROTO_ALL_SET = new HashSet<>();
-    public static final String DEFAULT_TLS_CIPHERS = "HIGH:!aNULL:!eNULL:!EXPORT:!DES:!RC4:!MD5:!kRSA";
+    public static final String DEFAULT_TLS_CIPHERS_12 = "HIGH:!aNULL:!eNULL:!EXPORT:!DES:!RC4:!MD5:!kRSA";
+    public static final String DEFAULT_TLS_CIPHERS_13 = "TLS_AES_256_GCM_SHA384:TLS_CHACHA20_POLY1305_SHA256:TLS_AES_128_GCM_SHA256";
+    /**
+     * Default cipher list for TLS 1.2 and below.
+     * @deprecated Replaced by {@link #DEFAULT_TLS_CIPHERS_12}
+     */
+    @Deprecated
+    public static final String DEFAULT_TLS_CIPHERS = DEFAULT_TLS_CIPHERS_12;
 
     static {
-        /* Default used if protocols are not configured, also used if
-         * protocols="All"
+        /*
+         * Default used if protocols are not configured, also used if protocols="All"
          */
         SSL_PROTO_ALL_SET.add(Constants.SSL_PROTO_SSLv2Hello);
         SSL_PROTO_ALL_SET.add(Constants.SSL_PROTO_TLSv1);
@@ -71,14 +78,15 @@ public class SSLHostConfig implements Serializable {
     }
 
     private Type configType = null;
+    private Type trustConfigType = null;
 
     private String hostName = DEFAULT_SSL_HOST_NAME;
 
-    private transient Long openSslConfContext = Long.valueOf(0);
+    private transient volatile Long openSslConfContext = Long.valueOf(0);
     // OpenSSL can handle multiple certs in a single config so the reference to
     // the context is here at the virtual host level. JSSE can't so the
     // reference is held on the certificate.
-    private transient Long openSslContext = Long.valueOf(0);
+    private transient volatile Long openSslContext = Long.valueOf(0);
 
     private boolean tls13RenegotiationAvailable = false;
 
@@ -91,21 +99,27 @@ public class SSLHostConfig implements Serializable {
     // Need to know if TLS 1.3 has been explicitly requested as a warning needs
     // to generated if it is explicitly requested for a JVM that does not
     // support it. Uses a set so it is extensible for TLS 1.4 etc.
-    private Set<String> explicitlyRequestedProtocols = new HashSet<>();
+    private final Set<String> explicitlyRequestedProtocols = new HashSet<>();
     // Nested
     private SSLHostConfigCertificate defaultCertificate = null;
-    private Set<SSLHostConfigCertificate> certificates = new LinkedHashSet<>(4);
+    private final Set<SSLHostConfigCertificate> certificates = new LinkedHashSet<>(4);
     // Common
     private String certificateRevocationListFile;
     private CertificateVerification certificateVerification = CertificateVerification.NONE;
     private int certificateVerificationDepth = 10;
     // Used to track if certificateVerificationDepth has been explicitly set
     private boolean certificateVerificationDepthConfigured = false;
-    private String ciphers;
+    private String ciphers = DEFAULT_TLS_CIPHERS_12;
+    private String cipherSuites = DEFAULT_TLS_CIPHERS_13;
     private LinkedHashSet<Cipher> cipherList = null;
+    private LinkedHashSet<Cipher> cipherSuiteList = null;
     private List<String> jsseCipherNames = null;
-    private String honorCipherOrder = null;
-    private Set<String> protocols = new HashSet<>();
+    private boolean honorCipherOrder = false;
+    private boolean ocspEnabled = false;
+    private boolean ocspSoftFail = true;
+    private int ocspTimeout = 15000;
+    private int ocspVerifyFlags = 0;
+    private final Set<String> protocols = new HashSet<>();
     // Values <0 mean use the implementation default
     private int sessionCacheSize = -1;
     private int sessionTimeout = 86400;
@@ -120,6 +134,8 @@ public class SSLHostConfig implements Serializable {
     private String truststoreProvider = System.getProperty("javax.net.ssl.trustStoreProvider");
     private String truststoreType = System.getProperty("javax.net.ssl.trustStoreType");
     private transient KeyStore truststore = null;
+    private String groups = null;
+    private LinkedHashSet<Group> groupList = null;
     // OpenSSL
     private String certificateRevocationListPath;
     private String caCertificateFile;
@@ -173,18 +189,40 @@ public class SSLHostConfig implements Serializable {
 
     /**
      * Set property which belongs to the specified configuration type.
-     * @param name the property name
+     *
+     * @param name       the property name
      * @param configType the configuration type
-     * @return true if the property belongs to the current configuration,
-     *   and false otherwise
+     *
+     * @return true if the property belongs to the current configuration type, and false otherwise
      */
     boolean setProperty(String name, Type configType) {
         if (this.configType == null) {
             this.configType = configType;
         } else {
             if (configType != this.configType) {
-                log.warn(sm.getString("sslHostConfig.mismatch",
-                        name, getHostName(), configType, this.configType));
+                log.warn(sm.getString("sslHostConfig.mismatch", name, getHostName(), configType, this.configType));
+                return false;
+            }
+        }
+        return true;
+    }
+
+
+    /**
+     * Set property which belongs to the specified trust configuration type.
+     *
+     * @param name            the property name
+     * @param trustConfigType the trust configuration type
+     *
+     * @return true if the property belongs to the current trust configuration type, and false otherwise
+     */
+    boolean setTrustProperty(String name, Type trustConfigType) {
+        if (this.trustConfigType == null) {
+            this.trustConfigType = trustConfigType;
+        } else {
+            if (trustConfigType != this.trustConfigType) {
+                log.warn(sm.getString("sslHostConfig.mismatch.trust", name, getHostName(), trustConfigType,
+                        this.trustConfigType));
                 return false;
             }
         }
@@ -238,8 +276,8 @@ public class SSLHostConfig implements Serializable {
 
     private void registerDefaultCertificate() {
         if (defaultCertificate == null) {
-            SSLHostConfigCertificate defaultCertificate = new SSLHostConfigCertificate(
-                    this, SSLHostConfigCertificate.Type.UNDEFINED);
+            SSLHostConfigCertificate defaultCertificate =
+                    new SSLHostConfigCertificate(this, SSLHostConfigCertificate.Type.UNDEFINED);
             addCertificate(defaultCertificate);
             this.defaultCertificate = defaultCertificate;
         }
@@ -249,14 +287,14 @@ public class SSLHostConfig implements Serializable {
     public void addCertificate(SSLHostConfigCertificate certificate) {
         // Need to make sure that if there is more than one certificate, none of
         // them have a type of undefined.
-        if (certificates.size() == 0) {
+        if (certificates.isEmpty()) {
             certificates.add(certificate);
             return;
         }
 
         if (certificates.size() == 1 &&
                 certificates.iterator().next().getType() == SSLHostConfigCertificate.Type.UNDEFINED ||
-                        certificate.getType() == SSLHostConfigCertificate.Type.UNDEFINED) {
+                certificate.getType() == SSLHostConfigCertificate.Type.UNDEFINED) {
             // Invalid config
             throw new IllegalArgumentException(sm.getString("sslHostConfig.certificate.notype"));
         }
@@ -276,7 +314,6 @@ public class SSLHostConfig implements Serializable {
         } else if (openSslConf != null) {
             throw new IllegalArgumentException(sm.getString("sslHostConfig.opensslconf.alreadySet"));
         }
-        setProperty("<OpenSSLConf>", Type.OPENSSL);
         openSslConf = conf;
     }
 
@@ -287,7 +324,7 @@ public class SSLHostConfig implements Serializable {
 
 
     public Set<SSLHostConfigCertificate> getCertificates(boolean createDefaultIfEmpty) {
-        if (certificates.size() == 0 && createDefaultIfEmpty) {
+        if (certificates.isEmpty() && createDefaultIfEmpty) {
             registerDefaultCertificate();
         }
         return certificates;
@@ -299,6 +336,12 @@ public class SSLHostConfig implements Serializable {
     // TODO: This certificate setter can be removed once it is no longer
     // necessary to support the old configuration attributes (Tomcat 10?).
 
+    /**
+     * @return The default certificate key password.
+     *
+     * @deprecated Obtain the prefered Certificate and call this method, there.
+     */
+    @Deprecated
     public String getCertificateKeyPassword() {
         if (defaultCertificate == null) {
             return null;
@@ -306,12 +349,47 @@ public class SSLHostConfig implements Serializable {
             return defaultCertificate.getCertificateKeyPassword();
         }
     }
+
+
+    /**
+     * @param certificateKeyPassword The password for the default certificate's key.
+     *
+     * @deprecated Obtain the prefered Certificate and call this method, there.
+     */
+    @Deprecated
     public void setCertificateKeyPassword(String certificateKeyPassword) {
         registerDefaultCertificate();
         defaultCertificate.setCertificateKeyPassword(certificateKeyPassword);
     }
 
 
+    /**
+     * @return The password for the default certificate's key.
+     *
+     * @deprecated Obtain the prefered Certificate and call this method, there.
+     */
+    @Deprecated
+    public String getCertificateKeyPasswordFile() {
+        if (defaultCertificate == null) {
+            return null;
+        } else {
+            return defaultCertificate.getCertificateKeyPasswordFile();
+        }
+    }
+
+
+    /**
+     * @param certificateKeyPasswordFile The file containing the password for the default certificate's key.
+     *
+     * @deprecated Obtain the prefered Certificate and call this method, there.
+     */
+    @Deprecated
+    public void setCertificateKeyPasswordFile(String certificateKeyPasswordFile) {
+        registerDefaultCertificate();
+        defaultCertificate.setCertificateKeyPasswordFile(certificateKeyPasswordFile);
+    }
+
+
     public void setCertificateRevocationListFile(String certificateRevocationListFile) {
         this.certificateRevocationListFile = certificateRevocationListFile;
     }
@@ -324,8 +402,7 @@ public class SSLHostConfig implements Serializable {
 
     public void setCertificateVerification(String certificateVerification) {
         try {
-            this.certificateVerification =
-                    CertificateVerification.fromString(certificateVerification);
+            this.certificateVerification = CertificateVerification.fromString(certificateVerification);
         } catch (IllegalArgumentException iae) {
             // If the specified value is not recognised, default to the
             // strictest possible option.
@@ -367,36 +444,59 @@ public class SSLHostConfig implements Serializable {
 
 
     /**
-     * Set the new cipher configuration. Note: Regardless of the format used to
-     * set the configuration, it is always stored in OpenSSL format.
+     * Set the new cipher (TLSv1.2 and below) configuration. Note: Regardless of the format used to set the
+     * configuration, it is always stored in OpenSSL format.
      *
      * @param ciphersList The new cipher configuration in OpenSSL or JSSE format
      */
     public void setCiphers(String ciphersList) {
         // Ciphers is stored in OpenSSL format. Convert the provided value if
         // necessary.
-        if (ciphersList != null && !ciphersList.contains(":")) {
-            StringBuilder sb = new StringBuilder();
-            // Not obviously in OpenSSL format. May be a single OpenSSL or JSSE
-            // cipher name. May be a comma separated list of cipher names
-            String ciphers[] = ciphersList.split(",");
-            for (String cipher : ciphers) {
-                String trimmed = cipher.trim();
-                if (trimmed.length() > 0) {
-                    String openSSLName = OpenSSLCipherConfigurationParser.jsseToOpenSSL(trimmed);
-                    if (openSSLName == null) {
-                        // Not a JSSE name. Maybe an OpenSSL name or alias
-                        openSSLName = trimmed;
+        if (ciphersList != null) {
+            if (ciphersList.contains(":")) {
+                // OpenSSL format
+                StringBuilder sb = new StringBuilder();
+                String[] components = ciphersList.split(":");
+                // Remove any TLS 1.3 cipher suites
+                for (String component : components) {
+                    String trimmed = component.trim();
+                    if (OpenSSLCipherConfigurationParser.isTls13Cipher(trimmed)) {
+                        log.warn(sm.getString("sslHostConfig.ignoreTls13Ciphersuite", trimmed));
+                    } else {
+                        if (sb.length() > 0) {
+                            sb.append(':');
+                        }
+                        sb.append(trimmed);
                     }
-                    if (sb.length() > 0) {
-                        sb.append(':');
-                    }
-                    sb.append(openSSLName);
                 }
+                this.ciphers = sb.toString();
+            } else {
+                // Not obviously in OpenSSL format. Might be a single OpenSSL or JSSE
+                // cipher name. Might be a comma separated list of cipher names
+                StringBuilder sb = new StringBuilder();
+                String[] ciphers = ciphersList.split(",");
+                for (String cipher : ciphers) {
+                    String trimmed = cipher.trim();
+                    if (!trimmed.isEmpty()) {
+                        if (OpenSSLCipherConfigurationParser.isTls13Cipher(trimmed)) {
+                            log.warn(sm.getString("sslHostConfig.ignoreTls13Ciphersuite", trimmed));
+                            continue;
+                        }
+                        String openSSLName = OpenSSLCipherConfigurationParser.jsseToOpenSSL(trimmed);
+                        if (openSSLName == null) {
+                            // Not a JSSE name. Maybe an OpenSSL name or alias
+                            openSSLName = trimmed;
+                        }
+                        if (sb.length() > 0) {
+                            sb.append(':');
+                        }
+                        sb.append(openSSLName);
+                    }
+                }
+                this.ciphers = sb.toString();
             }
-            this.ciphers = sb.toString();
         } else {
-            this.ciphers = ciphersList;
+            this.ciphers = null;
         }
         this.cipherList = null;
         this.jsseCipherNames = null;
@@ -407,14 +507,6 @@ public class SSLHostConfig implements Serializable {
      * @return An OpenSSL cipher string for the current configuration.
      */
     public String getCiphers() {
-        if (ciphers == null) {
-            if (!JreCompat.isJre8Available() && Type.JSSE.equals(configType)) {
-                ciphers = DEFAULT_TLS_CIPHERS + ":!DHE";
-            } else {
-                ciphers = DEFAULT_TLS_CIPHERS;
-            }
-
-        }
         return ciphers;
     }
 
@@ -428,26 +520,89 @@ public class SSLHostConfig implements Serializable {
 
 
     /**
-     * Obtain the list of JSSE cipher names for the current configuration.
-     * Ciphers included in the configuration but not supported by JSSE will be
-     * excluded from this list.
+     * Obtain the list of JSSE cipher names for the current configuration. Ciphers included in the configuration but not
+     * supported by JSSE will be excluded from this list.
      *
      * @return A list of the JSSE cipher names
      */
     public List<String> getJsseCipherNames() {
         if (jsseCipherNames == null) {
-            jsseCipherNames = OpenSSLCipherConfigurationParser.convertForJSSE(getCipherList());
+            Set<Cipher> jsseCiphers = new HashSet<>();
+            jsseCiphers.addAll(getCipherSuiteList());
+            jsseCiphers.addAll(getCipherList());
+            jsseCipherNames = OpenSSLCipherConfigurationParser.convertForJSSE(jsseCiphers);
         }
         return jsseCipherNames;
     }
 
 
-    public void setHonorCipherOrder(String honorCipherOrder) {
+    /**
+     * Set the cipher suite (TLSv1.3) configuration.
+     *
+     * @param cipherSuites The cipher suites to use in a colon-separated, preference order list
+     */
+    public void setCipherSuites(String cipherSuites) {
+        StringBuilder sb = new StringBuilder();
+        String[] values;
+        if (cipherSuites.contains(":")) {
+            // OpenSSL format
+            values = cipherSuites.split(":");
+        } else {
+            // JSSE format or possible a single cipher suite name
+            values = cipherSuites.split(",");
+        }
+        for (String value : values) {
+            String trimmed = value.trim();
+            if (!trimmed.isEmpty()) {
+                if (!OpenSSLCipherConfigurationParser.isTls13Cipher(trimmed)) {
+                    log.warn(sm.getString("sslHostConfig.ignoreNonTls13Ciphersuite", trimmed));
+                    continue;
+                }
+                /*
+                 * OpenSSL and JSSE names for TLSv1.3 cipher suites are currently (January 2026) the same but handle the
+                 * possible future case where they are not.
+                 */
+                String openSSLName = OpenSSLCipherConfigurationParser.jsseToOpenSSL(trimmed);
+                if (openSSLName == null) {
+                    // Not a JSSE name. Maybe an OpenSSL name or alias
+                    openSSLName = trimmed;
+                }
+                if (sb.length() > 0) {
+                    sb.append(':');
+                }
+                sb.append(trimmed);
+            }
+        }
+        this.cipherSuites = sb.toString();
+        this.cipherSuiteList = null;
+        this.jsseCipherNames = null;
+    }
+
+
+    /**
+     * Obtain the current cipher suite (TLSv1.3) configuration.
+     *
+     * @return An OpenSSL cipher suite string for the current configuration.
+     */
+    public String getCipherSuites() {
+        return cipherSuites;
+    }
+
+
+    private LinkedHashSet<Cipher> getCipherSuiteList() {
+        if (cipherSuiteList == null) {
+            cipherSuiteList = OpenSSLCipherConfigurationParser.parse(getCipherSuites());
+        }
+        return cipherSuiteList;
+    }
+
+
+    public void setHonorCipherOrder(boolean honorCipherOrder) {
         this.honorCipherOrder = honorCipherOrder;
     }
 
 
-    public String getHonorCipherOrder() {
+    public boolean getHonorCipherOrder() {
         return honorCipherOrder;
     }
 
@@ -458,14 +613,53 @@ public class SSLHostConfig implements Serializable {
 
 
     /**
-     * @return The host name associated with this SSL configuration - always in
-     *         lower case.
+     * @return The host name associated with this SSL configuration - always in lower case.
      */
     public String getHostName() {
         return hostName;
     }
 
 
+    public boolean getOcspEnabled() {
+        return ocspEnabled;
+    }
+
+
+    public void setOcspEnabled(boolean ocspEnabled) {
+        this.ocspEnabled = ocspEnabled;
+    }
+
+
+    public boolean getOcspSoftFail() {
+        return ocspSoftFail;
+    }
+
+
+    public void setOcspSoftFail(boolean ocspSoftFail) {
+        this.ocspSoftFail = ocspSoftFail;
+    }
+
+
+    public int getOcspTimeout() {
+        return ocspTimeout;
+    }
+
+
+    public void setOcspTimeout(int ocspTimeout) {
+        this.ocspTimeout = ocspTimeout;
+    }
+
+
+    public int getOcspVerifyFlags() {
+        return ocspVerifyFlags;
+    }
+
+
+    public void setOcspVerifyFlags(int ocspVerifyFlags) {
+        this.ocspVerifyFlags = ocspVerifyFlags;
+    }
+
+
     public void setProtocols(String input) {
         protocols.clear();
         explicitlyRequestedProtocols.clear();
@@ -481,7 +675,7 @@ public class SSLHostConfig implements Serializable {
 
         // Split using a positive lookahead to keep the separator in
         // the capture so we can check which case it is.
-        for (String value: input.split("(?=[-+,])")) {
+        for (String value : input.split("(?=[-+,])")) {
             String trimmed = value.trim();
             // Ignore token which only consists of prefix character
             if (trimmed.length() > 1) {
@@ -506,8 +700,7 @@ public class SSLHostConfig implements Serializable {
                         trimmed = trimmed.substring(1).trim();
                     }
                     if (!protocols.isEmpty()) {
-                        log.warn(sm.getString("sslHostConfig.prefix_missing",
-                                 trimmed, getHostName()));
+                        log.warn(sm.getString("sslHostConfig.prefix_missing", trimmed, getHostName()));
                     }
                     if (trimmed.equalsIgnoreCase(Constants.SSL_PROTO_ALL)) {
                         protocols.addAll(SSL_PROTO_ALL_SET);
@@ -551,11 +744,51 @@ public class SSLHostConfig implements Serializable {
     }
 
 
+    /**
+     * @return the configured named groups
+     */
+    public String getGroups() {
+        return groups;
+    }
+
+
+    /**
+     * Set the enabled named groups.
+     * @param groupsString the case sensitive comma separated list of groups
+     */
+    public void setGroups(String groupsString) {
+        if (groupsString != null) {
+            LinkedHashSet<Group> groupList = new LinkedHashSet<>();
+            String[] groupNames = groupsString.split(",");
+            for (String groupName : groupNames) {
+                Group group = Group.valueOf(groupName.trim());
+                groupList.add(group);
+            }
+            this.groups = groupsString;
+            this.groupList = groupList;
+        }
+    }
+
+
+    /**
+     * @return the groupList
+     */
+    public LinkedHashSet<Group> getGroupList() {
+        return this.groupList;
+    }
+
+
     // ---------------------------------- JSSE specific configuration properties
 
     // TODO: These certificate setters can be removed once it is no longer
     // necessary to support the old configuration attributes (Tomcat 10?).
 
+    /**
+     * @return The key alias for the default certificate key.
+     *
+     * @deprecated Obtain the prefered Certificate and call this method, there.
+     */
+    @Deprecated
     public String getCertificateKeyAlias() {
         if (defaultCertificate == null) {
             return null;
@@ -563,12 +796,25 @@ public class SSLHostConfig implements Serializable {
             return defaultCertificate.getCertificateKeyAlias();
         }
     }
+
+    /**
+     * @param certificateKeyAlias The alias of the certificate key.
+     *
+     * @deprecated Obtain the prefered Certificate and call this method, there.
+     */
+    @Deprecated
     public void setCertificateKeyAlias(String certificateKeyAlias) {
         registerDefaultCertificate();
         defaultCertificate.setCertificateKeyAlias(certificateKeyAlias);
     }
 
 
+    /**
+     * @return The keystore file for the default certificate.
+     *
+     * @deprecated Obtain the prefered Certificate and call this method, there.
+     */
+    @Deprecated
     public String getCertificateKeystoreFile() {
         if (defaultCertificate == null) {
             return null;
@@ -576,12 +822,25 @@ public class SSLHostConfig implements Serializable {
             return defaultCertificate.getCertificateKeystoreFile();
         }
     }
+
+    /**
+     * @param certificateKeystoreFile The file containing the certificate keystore.
+     *
+     * @deprecated Obtain the prefered Certificate and call this method, there.
+     */
+    @Deprecated
     public void setCertificateKeystoreFile(String certificateKeystoreFile) {
         registerDefaultCertificate();
         defaultCertificate.setCertificateKeystoreFile(certificateKeystoreFile);
     }
 
 
+    /**
+     * @return The password for the default certificate's keystore.
+     *
+     * @deprecated Obtain the prefered Certificate and call this method, there.
+     */
+    @Deprecated
     public String getCertificateKeystorePassword() {
         if (defaultCertificate == null) {
             return null;
@@ -589,12 +848,51 @@ public class SSLHostConfig implements Serializable {
             return defaultCertificate.getCertificateKeystorePassword();
         }
     }
+
+    /**
+     * @param certificateKeystorePassword The password for the certificate keystore.
+     *
+     * @deprecated Obtain the prefered Certificate and call this method, there.
+     */
+    @Deprecated
     public void setCertificateKeystorePassword(String certificateKeystorePassword) {
         registerDefaultCertificate();
         defaultCertificate.setCertificateKeystorePassword(certificateKeystorePassword);
     }
 
 
+    /**
+     * @return The file containing the default certificate's keystore password.
+     *
+     * @deprecated Obtain the prefered Certificate and call this method, there.
+     */
+    @Deprecated
+    public String getCertificateKeystorePasswordFile() {
+        if (defaultCertificate == null) {
+            return null;
+        } else {
+            return defaultCertificate.getCertificateKeystorePasswordFile();
+        }
+    }
+
+    /**
+     * @param certificateKeystorePasswordFile The file containing the default certificate's keystore password.
+     *
+     * @deprecated Obtain the prefered Certificate and call this method, there.
+     */
+    @Deprecated
+    public void setCertificateKeystorePasswordFile(String certificateKeystorePasswordFile) {
+        registerDefaultCertificate();
+        defaultCertificate.setCertificateKeystorePasswordFile(certificateKeystorePasswordFile);
+    }
+
+
+    /**
+     * @return The provider for the default certificate's keystore.
+     *
+     * @deprecated Obtain the prefered Certificate and call this method, there.
+     */
+    @Deprecated
     public String getCertificateKeystoreProvider() {
         if (defaultCertificate == null) {
             return null;
@@ -602,12 +900,25 @@ public class SSLHostConfig implements Serializable {
             return defaultCertificate.getCertificateKeystoreProvider();
         }
     }
+
+    /**
+     * @param certificateKeystoreProvider The provider for the default certificate's keystore.
+     *
+     * @deprecated Obtain the prefered Certificate and call this method, there.
+     */
+    @Deprecated
     public void setCertificateKeystoreProvider(String certificateKeystoreProvider) {
         registerDefaultCertificate();
         defaultCertificate.setCertificateKeystoreProvider(certificateKeystoreProvider);
     }
 
 
+    /**
+     * @return The type of the default certificate's keystore.
+     *
+     * @deprecated Obtain the prefered Certificate and call this method, there.
+     */
+    @Deprecated
     public String getCertificateKeystoreType() {
         if (defaultCertificate == null) {
             return null;
@@ -615,6 +926,13 @@ public class SSLHostConfig implements Serializable {
             return defaultCertificate.getCertificateKeystoreType();
         }
     }
+
+    /**
+     * @param certificateKeystoreType The type of the default certificate's keystore.
+     *
+     * @deprecated Obtain the prefered Certificate and call this method, there.
+     */
+    @Deprecated
     public void setCertificateKeystoreType(String certificateKeystoreType) {
         registerDefaultCertificate();
         defaultCertificate.setCertificateKeystoreType(certificateKeystoreType);
@@ -655,7 +973,7 @@ public class SSLHostConfig implements Serializable {
 
 
     public void setTrustManagerClassName(String trustManagerClassName) {
-        setProperty("trustManagerClassName", Type.JSSE);
+        setTrustProperty("trustManagerClassName", Type.JSSE);
         this.trustManagerClassName = trustManagerClassName;
     }
 
@@ -666,7 +984,7 @@ public class SSLHostConfig implements Serializable {
 
 
     public void setTruststoreAlgorithm(String truststoreAlgorithm) {
-        setProperty("truststoreAlgorithm", Type.JSSE);
+        setTrustProperty("truststoreAlgorithm", Type.JSSE);
         this.truststoreAlgorithm = truststoreAlgorithm;
     }
 
@@ -677,7 +995,7 @@ public class SSLHostConfig implements Serializable {
 
 
     public void setTruststoreFile(String truststoreFile) {
-        setProperty("truststoreFile", Type.JSSE);
+        setTrustProperty("truststoreFile", Type.JSSE);
         this.truststoreFile = truststoreFile;
     }
 
@@ -688,7 +1006,7 @@ public class SSLHostConfig implements Serializable {
 
 
     public void setTruststorePassword(String truststorePassword) {
-        setProperty("truststorePassword", Type.JSSE);
+        setTrustProperty("truststorePassword", Type.JSSE);
         this.truststorePassword = truststorePassword;
     }
 
@@ -699,7 +1017,7 @@ public class SSLHostConfig implements Serializable {
 
 
     public void setTruststoreProvider(String truststoreProvider) {
-        setProperty("truststoreProvider", Type.JSSE);
+        setTrustProperty("truststoreProvider", Type.JSSE);
         this.truststoreProvider = truststoreProvider;
     }
 
@@ -718,7 +1036,7 @@ public class SSLHostConfig implements Serializable {
 
 
     public void setTruststoreType(String truststoreType) {
-        setProperty("truststoreType", Type.JSSE);
+        setTrustProperty("truststoreType", Type.JSSE);
         this.truststoreType = truststoreType;
     }
 
@@ -742,6 +1060,7 @@ public class SSLHostConfig implements Serializable {
 
 
     public void setTrustStore(KeyStore truststore) {
+        setTrustProperty("trustStore", Type.JSSE);
         this.truststore = truststore;
     }
 
@@ -749,19 +1068,18 @@ public class SSLHostConfig implements Serializable {
     public KeyStore getTruststore() throws IOException {
         KeyStore result = truststore;
         if (result == null) {
-            if (truststoreFile != null){
+            if (truststoreFile != null) {
                 try {
-                    result = SSLUtilBase.getStore(getTruststoreType(), getTruststoreProvider(),
-                            getTruststoreFile(), getTruststorePassword());
+                    result = SSLUtilBase.getStore(getTruststoreType(), getTruststoreProvider(), getTruststoreFile(),
+                            getTruststorePassword(), null);
                 } catch (IOException ioe) {
                     Throwable cause = ioe.getCause();
                     if (cause instanceof UnrecoverableKeyException) {
                         // Log a warning we had a password issue
-                        log.warn(sm.getString("sslHostConfig.invalid_truststore_password"),
-                                cause);
+                        log.warn(sm.getString("sslHostConfig.invalid_truststore_password"), cause);
                         // Re-try
-                        result = SSLUtilBase.getStore(getTruststoreType(), getTruststoreProvider(),
-                                getTruststoreFile(), null);
+                        result = SSLUtilBase.getStore(getTruststoreType(), getTruststoreProvider(), getTruststoreFile(),
+                                null, null);
                     } else {
                         // Something else went wrong - re-throw
                         throw ioe;
@@ -785,6 +1103,7 @@ public class SSLHostConfig implements Serializable {
             return defaultCertificate.getCertificateChainFile();
         }
     }
+
     public void setCertificateChainFile(String certificateChainFile) {
         registerDefaultCertificate();
         defaultCertificate.setCertificateChainFile(certificateChainFile);
@@ -798,6 +1117,7 @@ public class SSLHostConfig implements Serializable {
             return defaultCertificate.getCertificateFile();
         }
     }
+
     public void setCertificateFile(String certificateFile) {
         registerDefaultCertificate();
         defaultCertificate.setCertificateFile(certificateFile);
@@ -811,6 +1131,7 @@ public class SSLHostConfig implements Serializable {
             return defaultCertificate.getCertificateKeyFile();
         }
     }
+
     public void setCertificateKeyFile(String certificateKeyFile) {
         registerDefaultCertificate();
         defaultCertificate.setCertificateKeyFile(certificateKeyFile);
@@ -829,7 +1150,7 @@ public class SSLHostConfig implements Serializable {
 
 
     public void setCaCertificateFile(String caCertificateFile) {
-        if (setProperty("caCertificateFile", Type.OPENSSL)) {
+        if (setTrustProperty("caCertificateFile", Type.OPENSSL)) {
             // Reset default JSSE trust store if not a JSSE configuration
             if (truststoreFile != null) {
                 truststoreFile = null;
@@ -845,7 +1166,7 @@ public class SSLHostConfig implements Serializable {
 
 
     public void setCaCertificatePath(String caCertificatePath) {
-        if (setProperty("caCertificatePath", Type.OPENSSL)) {
+        if (setTrustProperty("caCertificatePath", Type.OPENSSL)) {
             // Reset default JSSE trust store if not a JSSE configuration
             if (truststoreFile != null) {
                 truststoreFile = null;
@@ -922,12 +1243,12 @@ public class SSLHostConfig implements Serializable {
     public static String adjustRelativePath(String path) throws FileNotFoundException {
         // Empty or null path can't point to anything useful. The assumption is
         // that the value is deliberately empty / null so leave it that way.
-        if (path == null || path.length() == 0) {
+        if (path == null || path.isEmpty()) {
             return path;
         }
         String newPath = path;
         File f = new File(newPath);
-        if ( !f.isAbsolute()) {
+        if (!f.isAbsolute()) {
             newPath = System.getProperty(Constants.CATALINA_BASE_PROP) + File.separator + newPath;
             f = new File(newPath);
         }
@@ -959,31 +1280,25 @@ public class SSLHostConfig implements Serializable {
         }
 
         public boolean isOptional() {
-           return optional;
+            return optional;
         }
 
         public static CertificateVerification fromString(String value) {
-            if ("true".equalsIgnoreCase(value) ||
-                    "yes".equalsIgnoreCase(value) ||
-                    "require".equalsIgnoreCase(value) ||
+            if ("true".equalsIgnoreCase(value) || "yes".equalsIgnoreCase(value) || "require".equalsIgnoreCase(value) ||
                     "required".equalsIgnoreCase(value)) {
                 return REQUIRED;
-            } else if ("optional".equalsIgnoreCase(value) ||
-                    "want".equalsIgnoreCase(value)) {
+            } else if ("optional".equalsIgnoreCase(value) || "want".equalsIgnoreCase(value)) {
                 return OPTIONAL;
-            } else if ("optionalNoCA".equalsIgnoreCase(value) ||
-                    "optional_no_ca".equalsIgnoreCase(value)) {
+            } else if ("optionalNoCA".equalsIgnoreCase(value) || "optional_no_ca".equalsIgnoreCase(value)) {
                 return OPTIONAL_NO_CA;
-            } else if ("false".equalsIgnoreCase(value) ||
-                    "no".equalsIgnoreCase(value) ||
+            } else if ("false".equalsIgnoreCase(value) || "no".equalsIgnoreCase(value) ||
                     "none".equalsIgnoreCase(value)) {
                 return NONE;
             } else {
                 // Could be a typo. Don't default to NONE since that is not
                 // secure. Force user to fix config. Could default to REQUIRED
                 // instead.
-                throw new IllegalArgumentException(
-                        sm.getString("sslHostConfig.certificateVerificationInvalid", value));
+                throw new IllegalArgumentException(sm.getString("sslHostConfig.certificateVerificationInvalid", value));
             }
         }
     }
